<?php
/**
 * stanislav.elbl, 12.03.2021
 */

namespace RonPortal\Zamestnanci;

use Ron\Controls\FormField;
use Ron\Controls\FormRowArray;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\RonPortalForm;
use RonPortal\PracovniPomer;
use RonPortal\RonPortalDBRecord;
use RonPortal\User;
use Tracy\Dumper;

class NahlasenaZmenaForm extends RonPortalForm
{
    var $typZmeny = null;

    /** @var PracovniPomer */
    var $pp;

    /** @var FormField */
    var $fPoznamka;

    function __construct($database)
    {
        parent::__construct($database);

        $this->title = TextManager::_('nahlaseni_zmeny');
        $this ->typZmeny = $_GET['typzmeny'];

        // TODO: nacist TypNahlaseneZmeny a overit, jestli je povolena v Portalu
        // - Kod, Popis, SystemovaPolozka, VyzadovatSchvaleni, Nastaveni, PovolitVIntrawebu, Poznamka

//        if ($this ->formAction == 'new') {
//            $this ->formAction = 'edit';  // editace existujicich hodnot z databaze
//        }

        // TODO: nacist osobu a zkontrolovat prava

        $this ->addFormFields();

        $this ->fPoznamka = $this ->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka', 'NahlasenaZmena') ->setCaption('Poznamka')
            ->setSqlFieldName(null)
            ->setCode('field-poznamka');
    }


    function addFormFields()
    {
        switch ($this ->typZmeny) {
            case 'trvale' :
                $this ->addFieldsAdresa(true);
                break;

            case 'prechodne' :
                $this ->addFieldsAdresa(false);
                break;
        }
    }

    /**
     * Prida pole pro zmenu adresy osoby (trvale nebo prechodne)
     * @param $trvale
     */
    function addFieldsAdresa($trvale) {
        $code = $trvale ? 'trvale' : 'prechodne';

        $this ->sqlTableName = 'AdresaOsoby';

        $this ->addField(FormField::FIELD_TYPE_DATE)
            ->setFieldName('PlatnostOd', 'AdresaOsoby') ->setCaption('Platnost_od')
            ->setCode("field-{$code}-platnostod")
            ->setFieldId("zmena_{$code}_platnostod");

        $this ->addField()
            ->setFieldName('Ulice', 'AdresaOsoby') ->setCaption('Ulice')
            ->setMaxLength(60)
            ->setCode("field-{$code}-ulice")
            ->setFieldId("zmena_{$code}_ulice");

//            $rec->addStringField('Ulice', true, 40, $this ->nacteneHodnoty['Ulice']);
//            $rec->addStringField('Mesto', true, 40, $this ->nacteneHodnoty['Mesto']);
//            $rec->addStringField('Obec', true, 10, $this ->nacteneHodnoty['Obec']);
//            $rec->addStringField('PSC', true, 10, $this ->nacteneHodnoty['PSC']);

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName('Stat', 'AdresaOsoby')->setCaption('Stat')
            ->setCode("field-{$code}-stat")
            ->setFieldId("zmena_{$code}_stat")
            ->setSelectSql('SELECT Kod, Popis FROM Stat ORDER BY Popis')
            ->setFieldCssClass('form-control ronselect');

        // telefony
        $this ->addField()
            ->setFieldName('Telefon1', 'AdresaOsoby') ->setCaption('Telefon_1')
            ->setMaxLength(20)
            ->setCode("field-{$code}-telefon1")
            ->setFieldId("zmena_{$code}_telefon1");

        $this ->addField()
            ->setFieldName('Telefon2', 'AdresaOsoby') ->setCaption('Telefon_2')
            ->setMaxLength(20)
            ->setCode("field-{$code}-telefon2")
            ->setFieldId("zmena_{$code}_telefon2");

        $this ->addField()
            ->setFieldName('Mobil', 'AdresaOsoby') ->setCaption('Mobil')
            ->setMaxLength(20)
            ->setCode("field-{$code}-mobil")
            ->setFieldId("zmena_{$code}_mobil");

//            $rec->addStringField('E_mail', true, 60, $this ->nacteneHodnoty['E_Mail']);
//            $rec->addIntField('ZpusobDopravy', true, $this ->nacteneHodnoty['DopravaDoZamestnani']);
//            $rec->addField('VzdalenostDoZamestnani', 'float', true, 0, (float)$this ->nacteneHodnoty['VzdalenostDoZamestnani']);

    }


    function getSqlFromPart()
    {
        switch (strtolower($this ->typZmeny)) {
            case 'trvale' :
            case 'prechodne' :
                return 'AdresaOsoby';
        }

        return parent::getSqlFromPart();
    }


    function getWhereCondition()
    {
        $params = $this ->database ->createParameters();

        $trvale = 0;
        switch (strtolower($this ->typZmeny)) {
            /** @noinspection PhpMissingBreakStatementInspection */
            case 'trvale' :
                $trvale = 1;
                // POKRACUJE DALE!
            case 'prechodne' :
                return
                    new SqlStatement(
                        'RC = ' . $params ->addParameter($this ->pp ->rc) .
                        ' AND TrvaleBydliste = ' . $params ->addParameter($trvale) .
                        ' AND (PlatnostDo IS NULL  OR  PlatnostDo >= ' . $params ->addParameter($this ->database ->dateToParam(time())) . ')',
                        $params);
        }

        return parent::getWhereCondition();
    }


    function loadRecord()
    {
        if ($this ->isInsert()) {
            $this->pp = PracovniPomer::nactiPP($this->database, (int)$_GET['osoba']);

            return parent::loadRecord();
        }

        // Nacist data ze zaznamu NahlasenaZmena
        // - id, TypZmeny, RC, DatumCas, Stav, Hodnoty, DatumCasZpracovani, Zpracoval, Nahlasil, Poznamka
        $params = $this ->database ->createParameters();
        $row = DBUtils::getSqlValue($this ->database,
            'SELECT TypZmeny, RC, DatumCas, Stav, Hodnoty, DatumCasZpracovani, Zpracoval, Nahlasil, Poznamka' .
            ' FROM NahlasenaZmena' .
            ' WHERE Id = ' . $params ->addParameter((int)$_GET['record']),
            $params, true);

        if (!$row) {
            return null;
        }

        $this ->typZmeny = $row[0];

        $hodnoty = explode("\n", $row[4]);
        // TODO: zpracovat nactene hodnoty a nakopirovat je do poli
        var_dump($row);

        return $row;
    }


    function loadOriginalValues()
    {
        $this ->data = new FormRowArray($this ->fields, $this ->loadRecord());
    }


    /**
     * Nacte do asociativniho pole data odeslana uzivatelem
     * @return array
     * @throws \Ron\RonException
     */
    private function sestavDataNahlaseneZmeny()
    {
        $result = [];

        foreach ($this->fields as $field) {
            $fieldTableName = $field ->tableAlias ?: $field ->tableName;

            if ($field ->visible  &&   $field ->fieldName  &&  $fieldTableName != 'NahlasenaZmena') {
                $result[$field ->fieldName] = $field ->getPostValue(false);
            }
        }

        return $result;
    }


    public function insertRecord()
    {
        if (!$this->validate(false)) {
            return false;
        }

        $pp = PracovniPomer::nactiPP($this ->database, (int)$_GET['osoba'] ?? 0);
        $insertData = $this ->sestavDataNahlaseneZmeny();
        $dataStr = '';
        foreach ($insertData as $key => $value) {
            $dataStr .= $key . '=' . $value . "\n";
        }

        $rec = new RonPortalDBRecord($this ->database, 'NahlasenaZmena', $this);
        $rec ->addField('Id', 'serial', true);
        $rec ->addStringField('TypZmeny', true, 0, $this ->typZmeny);
        $rec ->addStringField('RC', true, 15, $pp ->rc);
        $rec ->addTimestampField('DatumCas', true, time());
        $rec ->addIntField('Stav', true, 0);
        $rec ->addMemoField('Hodnoty', false, $dataStr);
        $rec ->addStringField('Nahlasil', true, 15, User::getRC());

        if ($this ->fPoznamka ->visible) {
            $f = $this ->fPoznamka ->createDBRecordField($this ->database, 'NahlasenaZmena', true, false);
            if ($f) {
                $rec ->addDBRecordField($f);
            }
        }

        // TODO: BEFORE_POST XML
        $rec ->insertRecord();
        // TODO: AFTER_POST XML
    }


    function handleAction()
    {
        if ($this ->formAction == 'new') {
            $this ->loadOriginalValues();
        }

        $result = parent::handleAction();


    }
}
